package ru.dbotthepony.mc.otm.block.entity.storage

import net.minecraft.core.BlockPos
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.mc.otm.block.entity.MatteryPoweredBlockEntity
import ru.dbotthepony.mc.otm.graph.storage.StorageNode
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.drive.IMatteryDrive
import ru.dbotthepony.mc.otm.capability.energy.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.config.MachinesConfig
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.menu.storage.DriveRackMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.storage.*
import ru.dbotthepony.mc.otm.storage.powered.PoweredVirtualComponent

class DriveRackBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) :
	MatteryPoweredBlockEntity(MBlockEntities.DRIVE_RACK, p_155229_, p_155230_) {
	val energy = WorkerEnergyStorage(this::setChangedLight, MachinesConfig.DRIVE_RACK)

	val container: MatteryContainer = object : MatteryContainer(this::setChanged, 4) {
		override fun setChanged(slot: Int, new: ItemStack, old: ItemStack) {
			super.setChanged(slot, new, old)

			// generics is going apeshit since storage types are invariant,
			// but since we don't know generics of upvalue mattery drive, its storage type
			// is defined as out variant
			old.getCapability(MatteryCapability.DRIVE).ifPresent {
				cell.computeIfAbsent(it.storageType as StorageStack.Type<ItemStorageStack>) {
					PoweredVirtualComponent(VirtualComponent(it), ::energy)
				}.remove(it as IMatteryDrive<ItemStorageStack>)
			}

			new.getCapability(MatteryCapability.DRIVE).ifPresent {
				cell.computeIfAbsent(it.storageType as StorageStack.Type<ItemStorageStack>) {
					PoweredVirtualComponent(VirtualComponent(it), ::energy)
				}.add(it as IMatteryDrive<ItemStorageStack>)
			}
		}
	}.also(::addDroppableContainer)

	val cell = StorageNode(energy)

	init {
		savetable(::energy, ENERGY_KEY)
		savetable(::container, INVENTORY_KEY)
		exposeEnergyGlobally(energy)
		exposeGlobally(MatteryCapability.STORAGE_NODE, cell)
	}

	override fun setLevel(level: Level) {
		super.setLevel(level)
		cell.discover(this)
	}

	override fun tick() {
		super.tick()
		cell.tickEnergyDemanding()
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return DriveRackMenu(containerID, inventory, this)
	}

	override fun setRemoved() {
		super.setRemoved()
		cell.isValid = false
	}
}
